@@ -26,9 +26,27 @@
 #[macro_use]
 extern crate lazy_static;
 
+#[cfg(test)]
+extern crate quickcheck;
+#[cfg(test)]
+#[macro_use(quickcheck)]
+extern crate quickcheck_macros;
+
 pub use cmd::{async_lint, lint};
 pub use model::{Code, Error, Lint, LintError, Lints, Problem, CONFIG_KEY_PREFIX};
 
 mod checks;
 mod cmd;
 mod model;
+
+#[cfg(doctest)]
+mod test_readme {
+    macro_rules! external_doc_test {
+        ($x:expr) => {
+            #[doc = $x]
+            extern "C" {}
+        };
+    }
+
+    external_doc_test!(include_str!("../README.md"));
+}
