@@ -2,6 +2,8 @@ use std::convert::TryInto;
 
 use miette::Diagnostic;
 use mit_commit::CommitMessage;
+use quickcheck::{Arbitrary, Gen};
+use strum_macros::EnumIter;
 use thiserror::Error;
 
 use crate::{
@@ -11,7 +13,7 @@ use crate::{
 };
 
 /// The lints that are supported
-#[derive(Debug, Eq, PartialEq, Copy, Clone, Hash, Ord, PartialOrd)]
+#[derive(Debug, Eq, PartialEq, Copy, Clone, Hash, Ord, PartialOrd, EnumIter)]
 pub enum Lint {
     /// Check for duplicated trailers
     ///
@@ -506,10 +508,10 @@ impl std::convert::TryFrom<&str> for Lint {
     type Error = Error;
 
     fn try_from(from: &str) -> Result<Self, Self::Error> {
-        Lint::all_lints()
-            .zip(Lint::all_lints().map(|lint| format!("{}", lint)))
-            .filter_map(|(lint, name): (Lint, String)| if name == from { Some(lint) } else { None })
-            .collect::<Vec<Lint>>()
+        Self::all_lints()
+            .zip(Self::all_lints().map(|lint| format!("{}", lint)))
+            .filter_map(|(lint, name): (Self, String)| if name == from { Some(lint) } else { None })
+            .collect::<Vec<Self>>()
             .first()
             .copied()
             .ok_or_else(|| Error::new_lint_not_found(from.into()))
@@ -531,7 +533,7 @@ impl From<Lint> for &str {
 impl Lint {
     /// Get an lint's unique name
     #[must_use]
-    pub fn name(self) -> &'static str {
+    pub const fn name(self) -> &'static str {
         match self {
             Lint::DuplicatedTrailers => checks::duplicate_trailers::CONFIG,
             Lint::PivotalTrackerIdMissing => checks::missing_pivotal_tracker_id::CONFIG,
@@ -573,6 +575,7 @@ lazy_static! {
         Lint::BodyWiderThan72Characters,
     ];
 }
+
 impl Lint {
     /// Iterator over all the lints
     ///
@@ -582,7 +585,7 @@ impl Lint {
     /// use mit_lint::Lint;
     /// assert!(Lint::all_lints().next().is_some())
     /// ```
-    pub fn all_lints() -> impl Iterator<Item = Lint> {
+    pub fn all_lints() -> impl Iterator<Item = Self> {
         ALL_LINTS.iter().copied()
     }
 
@@ -595,8 +598,8 @@ impl Lint {
     /// assert!(Lint::iterator().next().is_some())
     /// ```
     #[deprecated(since = "0.1.5", note = "iterator was an unusual name. Use all_lints")]
-    pub fn iterator() -> impl Iterator<Item = Lint> {
-        Lint::all_lints()
+    pub fn iterator() -> impl Iterator<Item = Self> {
+        Self::all_lints()
     }
 
     /// Check if a lint is enabled by default
@@ -682,66 +685,29 @@ impl Lint {
     ///
     /// # Errors
     /// If the lint does not exist
-    pub fn from_names(names: Vec<&str>) -> Result<Vec<Lint>, model::lints::Error> {
+    pub fn from_names(names: Vec<&str>) -> Result<Vec<Self>, model::lints::Error> {
         let lints: Lints = names.try_into()?;
         Ok(lints.into_iter().collect())
     }
 }
 
-#[cfg(test)]
-mod tests_lints {
-    use std::convert::TryInto;
-
-    use crate::model::Lint;
-
-    #[test]
-    fn it_is_convertible_to_string() {
-        let string: String = Lint::PivotalTrackerIdMissing.into();
-        assert_eq!("pivotal-tracker-id-missing".to_string(), string);
+impl Arbitrary for Lint {
+    fn arbitrary(g: &mut Gen) -> Self {
+        *g.choose(&ALL_LINTS.iter().copied().collect::<Vec<_>>())
+            .unwrap()
     }
 
-    #[test]
-    fn it_can_be_created_from_string() {
-        let lint: Lint = "pivotal-tracker-id-missing".try_into().unwrap();
-        assert_eq!(Lint::PivotalTrackerIdMissing, lint);
-    }
+    fn shrink(&self) -> Box<dyn Iterator<Item = Self>> {
+        let index = ALL_LINTS.iter().position(|other| self.eq(other));
 
-    #[test]
-    fn it_is_printable() {
-        assert_eq!(
-            "pivotal-tracker-id-missing",
-            &format!("{}", Lint::PivotalTrackerIdMissing)
-        );
-    }
-
-    #[test]
-    fn i_can_get_all_the_lints() {
-        let all: Vec<Lint> = Lint::all_lints().collect();
-        assert_eq!(
-            all,
-            vec![
-                Lint::DuplicatedTrailers,
-                Lint::PivotalTrackerIdMissing,
-                Lint::JiraIssueKeyMissing,
-                Lint::SubjectNotSeparateFromBody,
-                Lint::GitHubIdMissing,
-                Lint::SubjectLongerThan72Characters,
-                Lint::SubjectNotCapitalized,
-                Lint::SubjectEndsWithPeriod,
-                Lint::BodyWiderThan72Characters,
-                Lint::NotConventionalCommit,
-                Lint::NotEmojiLog,
-            ]
-        );
-    }
-
-    #[test]
-    fn i_can_get_if_a_lint_is_enabled_by_default() {
-        assert!(Lint::DuplicatedTrailers.enabled_by_default());
-        assert!(!Lint::PivotalTrackerIdMissing.enabled_by_default());
-        assert!(!Lint::JiraIssueKeyMissing.enabled_by_default());
-        assert!(Lint::SubjectNotSeparateFromBody.enabled_by_default());
-        assert!(!Lint::GitHubIdMissing.enabled_by_default());
+        match index {
+            None | Some(0) => quickcheck::empty_shrinker(),
+            Some(index) => ALL_LINTS
+                .get(index - 1)
+                .map_or(quickcheck::empty_shrinker(), |item| {
+                    quickcheck::single_shrinker(*item)
+                }),
+        }
     }
 }
 
@@ -764,8 +730,8 @@ pub enum Error {
 }
 
 impl Error {
-    fn new_lint_not_found(missing_lint: String) -> Error {
+    fn new_lint_not_found(missing_lint: String) -> Self {
         let length = missing_lint.len();
-        Error::LintNotFound(missing_lint, (0, length))
+        Self::LintNotFound(missing_lint, (0, length))
     }
 }
